export const ROUTES = {
  API_DOWNLOAD_MODEL: "/api/models",
  API_INFERENCE_IMAGE: "/api/render-image/inference",
  API_SUPPORT_IMAGE: "/api/render-image/support",
  DASHBOARD: "/dashboard",
  DEMO: "/demo",
  DOWNLOAD_PAGE: "/download",
  LANDING: "/",
  MODEL_SUMMARY_PAGE: "/model-summary-page",
  SETTINGS: "/settings",
  TRAINING: "/training",
  RETRAINING: "/training?retrainModelName="
}