# Copyright (c) 2023 Massachusetts Institute of Technology
# SPDX-License-Identifier: MIT
query GetPrototypeSupportEmbeddings($modelName:String!) {
  getPrototypeSupportEmbeddings(modelName:$modelName) {
    label
    prototype
    trainingExamples {
      coordinates
      inputData {
        file
        dataIndex
      }
      labels {
        label
        confidence
      }
      ood
    }
  }
}