# Copyright (c) 2023 Massachusetts Institute of Technology
# SPDX-License-Identifier: MIT
query ModelSummary($modelName: String!) {
  modelSummary(modelName: $modelName) {
    dateTrained
    lastModified
    modelType
    numTrainExamples {
      label
      numExamples
    }
  }
}
