# Copyright (c) 2023 Massachusetts Institute of Technology
# SPDX-License-Identifier: MIT
mutation RunInference(
  $modelName: String!,
  $sampleFilenames:[String]!
) {
  runInference(
    modelName:$modelName,
    sampleFilenames:$sampleFilenames
  ) {
    runId
    samples {
      coordinates
      inputData {
        file
        dataIndex
      }
      labels {
        label
        confidence
      }
      ood
    }
    version
  }
}