query RenderInferenceFeatureData($runId: Int!, $modelName: String!, $dataIndex: Int!) {
  renderInferenceFeatureData(runId: $runId, modelName: $modelName, dataIndex: $dataIndex){
    featureData,
    columnHeaders
  }
}