// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
export const COLORS = {
  BLUE: "#3366CC",

  GREEN: "#109618",
  MEDIUM_GREEN: "#28B463",
  LIGHT_GREEN: "#D5F5E3",

  ORANGE: "#FF9900",

  PURPLE: "#990099",

  RED: "#DC3912",
  PINK: "#E74C3C",
  LIGHT_PINK: "#FADBD8",
}

export const COLOR_BLIND_COLORS = {
  BLUE: "#235FA4",

  GREEN: "#6FDE6E",
  MEDIUM_GREEN: "#6FDE6E",
  LIGHT_GREEN: "#9FDD9E",

  ORANGE: "#E8F086",

  PURPLE: "#A691AE",

  RED: "#FF4242",
  PINK: "#FF4242",
  LIGHT_PINK: "#FFBFBF",
}