// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT

.uq-viz-container {
  width: 100%;
  display: flex;

  img {
    width: 100%;
  }

  span.circle {
    margin-top: 2px;
    margin-bottom: -2px;
    height: 1rem;
    width: 1rem;
    border: 1.5px solid black;
    border-radius: 50%;
    display: inline-block;
  }

  :global(.customTooltip.tooltip) {
    position: fixed;
    transform: translate(-50%,calc(-100% - 10px));
    opacity: 1;
  
    .arrow {
      left: 50%;
      transform: translate(-50%,0);
    }

    p {
      margin: 0;
    }

    table {
      width: 100%;
    }
  }

  .prototype {
    transition: 0.5s;

    &:hover {
      r: 7;
    }
  }

  pre {
    color: white;
    text-align: left;
    background-color: #333;
  }

  .uq-viz-sidebar {
    width: 200px;
    border: 1px solid gray;
  }
}