.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_loadEquineModelFile__0ujmu{display:none}.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_uploadModelContainer__2GX4J{display:flex;align-items:center;margin-top:.5rem}.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_uploadModelContainer__2GX4J p{margin-top:.25rem}#Training_training__xSvop{height:100%;padding:1em}#Training_training__xSvop h3{text-align:center}#Training_training__xSvop #Training_labelsContainer__3kqkP{padding-top:1em;padding-left:.5em;padding-right:.5em}#Training_training__xSvop #Training_labelsContainer__3kqkP .Training_row__za_5d>div{padding:.5em}#Training_training__xSvop #Training_submitButtonContainer__W821Z{text-align:center}#Training_trainSuccess__QB1bd{height:100%;text-align:center;display:flex;justify-content:center;align-items:center}.UploadSampleFiles_uploadSampleFiles__IgJM3 #UploadSampleFiles_loadSampleFiles__HF_Vw{display:none}.UploadSampleFiles_uploadSampleFiles__IgJM3 .UploadSampleFiles_icon__xt42X{float:right}.InfoTooltip_infoTooltip__XHOEb{cursor:pointer}