self.__BUILD_MANIFEST=function(s,c,a,e,t,d,i){return{__rewrites:{afterFiles:[],beforeFiles:[],fallback:[]},"/":[s,e,t,"static/css/ef9f5150e4d50df1.css","static/chunks/pages/index-a49c846d8bb28207.js"],"/_error":["static/chunks/pages/_error-7a92967bea80186d.js"],"/dashboard":[s,c,d,"static/chunks/581-923161b0fabfcdba.js",a,i,"static/css/2dcd9715a0fb2885.css","static/chunks/pages/dashboard-fc086b1aa7243427.js"],"/demo":[s,c,d,a,i,"static/css/eceecebb692aa855.css","static/chunks/pages/demo-e4909671c317b822.js"],"/download":[s,"static/css/fb1ec0dcf088682f.css","static/chunks/pages/download-d77cebf8e40c79cb.js"],"/model-summary-page":[s,c,a,"static/css/ba2b0b21e17b7ae6.css","static/chunks/pages/model-summary-page-d5b262c3e40eb950.js"],"/settings":["static/chunks/pages/settings-8e2a3eada448b8cb.js"],"/training":[s,c,e,t,"static/css/943c3034c4b068f0.css","static/chunks/pages/training-d1df3b6374d63123.js"],sortedPages:["/","/_app","/_error","/dashboard","/demo","/download","/model-summary-page","/settings","/training"]}}("static/chunks/192-b92151c79b9841d6.js","static/chunks/420-1522028cbdda33f5.js","static/chunks/996-561b4e3381be093c.js","static/chunks/743-9eb63513b6150d5f.js","static/chunks/402-363f0764365c8a98.js","static/chunks/644-6d3655bc34624a79.js","static/chunks/424-3619d8f510cf6772.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();