// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
.progressBarContainer {
  height: 1em;
  background: #ccc;
  position: relative;
  border-radius: 3px;

  .progressBar {
    border-radius: 3px;
    height: 100%;
    background: #007bff;
  }
}
