// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
@import "~/utils/_variables";

.demoSlidersContainer {
  input {
    cursor: not-allowed;
    pointer-events: none;
  }
}

#filteredTable {
  .filteredTableNoData {
    width: 100%;
    text-align: center;
  }

  div:first-child {
    overflow: unset;
    white-space: unset;
  }
}

:global(.dark) #filteredTable .filteredTableNoData {
  width: 100%;
  text-align: center;
  background-color: $box-dark-color;
}