// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
#training {
  height: 100%;
  padding: 1em;

  h3 {
    text-align: center;
  }

  #labelsContainer {
    padding-top: 1em;
    padding-left: 0.5em;
    padding-right: 0.5em;

    .row {
      > div {
        padding: 0.5em;
      }
    }
  }

  #submitButtonContainer {
    text-align: center;
  } 
}

#trainSuccess {
  height: 100%;
  text-align: center;
  display: flex;
  justify-content: center;
  align-items: center;
}