# Copyright (c) 2023 Massachusetts Institute of Technology
# SPDX-License-Identifier: MIT
type UploadFileResult {
    success: Boolean!
}

extend type Mutation {
    uploadFile(file: Upload!): UploadFileResult
}