// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
#controlBar {
  display: flex;
  justify-content: space-between;

  #confidenceThresholdsContainer {
    display: flex;
    flex-direction: column;
    width: 100%;

    input[type=range] {
      width: calc(100% - 3.5em);
    }
  }
}
