// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
import React from 'react'

export default function NoDataMessage() {
  return <div style={{textAlign: "center"}}><b>There is no data to show</b></div>
}