// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
.coloredLabel {
  padding: 0.2em;
  padding-left: 0.4em;
  padding-right: 0.4em;
  color: white;
  font-weight: bold;
  display: inline-block;
  margin-top: 1px;
  margin-bottom: 1px;
}