// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
#modelSummaryEmptyPage {
  display: flex;
  justify-content: center;
  align-items: center;
  height: 100%;
}
