// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
export default function setDocumentTitle(subTitle:string) {
  document.title = `EQUI(NE)\u{00B2} - ${subTitle}`
}
