// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
const LOCAL_STORAGE_KEYS = {
  colorBlindMode: "colorBlindMode",
  darkMode: "darkMode",
  serverUrl: "serverUrl",
} as const

export default LOCAL_STORAGE_KEYS
