#ControlBar_controlBar__3bTG_{display:flex;justify-content:space-between}#ControlBar_controlBar__3bTG_ #ControlBar_confidenceThresholdsContainer__pExtV{display:flex;flex-direction:column;width:100%}#ControlBar_controlBar__3bTG_ #ControlBar_confidenceThresholdsContainer__pExtV input[type=range]{width:calc(100% - 3.5em)}#Dashboard_dashboard__aZX5i #Dashboard_isData__A1bE1{padding-top:.1em;padding-bottom:.1em;background-color:#ddd}#Dashboard_dashboard__aZX5i #Dashboard_downloadButton__dZg8N{float:right}.dark #Dashboard_dashboard__aZX5i #Dashboard_isData__A1bE1{background:#111}#Dashboard_emptyDashboard__GSRzc{display:flex;justify-content:center;align-items:center;min-height:100vh;text-align:center}#Dashboard_emptyDashboard__GSRzc #Dashboard_loadDataFile__Ekz6e{display:none}#Filters_filters__ndzMp .Filters_selectDeselectContainer__VEiZK{font-size:.8em;margin-bottom:.5em}#Filters_filters__ndzMp .Filters_selectDeselectContainer__VEiZK .Filters_selectDeselectOption__vcQUE{cursor:pointer;display:inline-block;transition:.5s}#Filters_filters__ndzMp .Filters_selectDeselectContainer__VEiZK .Filters_selectDeselectOption__vcQUE:hover{text-decoration:underline}#Filters_filters__ndzMp .Filters_selectDeselectContainer__VEiZK .Filters_selectDeselectOption__vcQUE:active{transform:scale(.9)}.dark #Filters_filters__ndzMp .Filters_selectDeselectContainer__VEiZK .Filters_selectDeselectOption__vcQUE{color:#fff}.InfoTooltip_infoTooltip__XHOEb{cursor:pointer}.ScatterUQ_uq-viz-container__Nsp3M{width:100%;display:flex}.ScatterUQ_uq-viz-container__Nsp3M img{width:100%}.ScatterUQ_uq-viz-container__Nsp3M span.ScatterUQ_circle__GDFw0{margin-top:2px;margin-bottom:-2px;height:1rem;width:1rem;border:1.5px solid #000;border-radius:50%;display:inline-block}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip{position:fixed;transform:translate(-50%,calc(-100% - 10px));opacity:1}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip .ScatterUQ_arrow__oXuJU{left:50%;transform:translate(-50%)}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip p{margin:0}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip table{width:100%}.ScatterUQ_uq-viz-container__Nsp3M .ScatterUQ_prototype__Si4Xc{transition:.5s}.ScatterUQ_uq-viz-container__Nsp3M .ScatterUQ_prototype__Si4Xc:hover{r:7}.ScatterUQ_uq-viz-container__Nsp3M pre{color:#fff;text-align:left;background-color:#333}.ScatterUQ_uq-viz-container__Nsp3M .ScatterUQ_uq-viz-sidebar__PeHXh{width:200px;border:1px solid gray}#FilteredTable_filteredTable__nDIdZ .FilteredTable_filteredTableNoData__M5RBR{width:100%;text-align:center}#FilteredTable_filteredTable__nDIdZ div:first-child{overflow:unset;white-space:unset}.dark #FilteredTable_filteredTable__nDIdZ .FilteredTable_filteredTableNoData__M5RBR{width:100%;text-align:center;background-color:#333}.SamplesBarChart_samplesBarChartContainer__HLURF .SamplesBarChart_legend__FYtRg{font-size:.8em;text-align:center}.SamplesBarChart_samplesBarChartContainer__HLURF .SamplesBarChart_legend__FYtRg .SamplesBarChart_labelColorBox__O1E22{display:inline-block;width:40px;height:1em;margin-left:1.5em;margin-bottom:-2px}