// Copyright (c) 2023 Massachusetts Institute of Technology
// SPDX-License-Identifier: MIT
export const darkModeLibraryOptions = {
  plugins: {
    legend: {
      labels: {
        color: "white",
      },
      title: {
        color: "white",
      }
    }
  },
  scales: {
    x: {
      grid: { borderColor: "#777", color: "#777" },
      ticks: { color: "white" },
    },
    y: {
      grid: { borderColor: "#777", color: "#777" },
      ticks: { color: "white" },
    },
  },
}
