.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_loadEquineModelFile__0ujmu{display:none}.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_uploadModelContainer__2GX4J{display:flex;align-items:center;margin-top:.5rem}.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_uploadModelContainer__2GX4J p{margin-top:.25rem}.UploadSampleFiles_uploadSampleFiles__IgJM3 #UploadSampleFiles_loadSampleFiles__HF_Vw{display:none}.UploadSampleFiles_uploadSampleFiles__IgJM3 .UploadSampleFiles_icon__xt42X{float:right}#SettingsForm_settingsForm__XFjnM h2{text-align:center}#Landing_landing__Pwkt_{display:flex;flex-direction:column;align-items:center}#Landing_landing__Pwkt_ img{margin-top:1em;width:50%;background-color:#fff}#Landing_landing__Pwkt_ #Landing_landingSettingsFormContainer__NfCZz{margin-top:3em;width:50%}.dark #Landing_landing__Pwkt_{background-color:#222}