# Copyright (c) 2023 Massachusetts Institute of Technology
# SPDX-License-Identifier: MIT
type Models {
    name: String!
    lastModified: Float!
}

extend type Query {
    models(extension:String): [Models!]!
}