#DownloadPage_downloadPage__KQmqv{padding:1em}#DownloadPage_downloadPage__KQmqv table{margin-left:-.5em;margin-right:-.5em}#DownloadPage_downloadPage__KQmqv table td,#DownloadPage_downloadPage__KQmqv table th{padding:.25em .5em}