# Copyright (c) 2023 Massachusetts Institute of Technology
# SPDX-License-Identifier: MIT
scalar Upload

type RunPipelineResult {
    runId: Int!
    samples: [Sample!]!
    version: String!,
}

type Mutation {
    runInference(
        modelName: String!,
        sampleFilenames: [String]!
    ): RunPipelineResult!

}