.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_loadEquineModelFile__0ujmu{display:none}.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_uploadModelContainer__2GX4J{display:flex;align-items:center;margin-top:.5rem}.SelectOrUploadModel_selectOrUploadModel___XYve #SelectOrUploadModel_uploadModelContainer__2GX4J p{margin-top:.25rem}#ModelSummaryEmptyPage_modelSummaryEmptyPage__lTZYS{display:flex;justify-content:center;align-items:center;height:100%}.InfoTooltip_infoTooltip__XHOEb{cursor:pointer}.ScatterUQ_uq-viz-container__Nsp3M{width:100%;display:flex}.ScatterUQ_uq-viz-container__Nsp3M img{width:100%}.ScatterUQ_uq-viz-container__Nsp3M span.ScatterUQ_circle__GDFw0{margin-top:2px;margin-bottom:-2px;height:1rem;width:1rem;border:1.5px solid #000;border-radius:50%;display:inline-block}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip{position:fixed;transform:translate(-50%,calc(-100% - 10px));opacity:1}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip .ScatterUQ_arrow__oXuJU{left:50%;transform:translate(-50%)}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip p{margin:0}.ScatterUQ_uq-viz-container__Nsp3M .customTooltip.tooltip table{width:100%}.ScatterUQ_uq-viz-container__Nsp3M .ScatterUQ_prototype__Si4Xc{transition:.5s}.ScatterUQ_uq-viz-container__Nsp3M .ScatterUQ_prototype__Si4Xc:hover{r:7}.ScatterUQ_uq-viz-container__Nsp3M pre{color:#fff;text-align:left;background-color:#333}.ScatterUQ_uq-viz-container__Nsp3M .ScatterUQ_uq-viz-sidebar__PeHXh{width:200px;border:1px solid gray}#ModelSummaryPage_modelSummaryPage__w9xEy{background-color:#eee;padding-top:.1em;padding-bottom:.1em;min-height:100vh}#ModelSummaryPage_modelSummaryPage__w9xEy h2{margin-top:0}#ModelSummaryPage_modelSummaryPage__w9xEy .ModelSummaryPage_trainingLabel__RfshS{height:calc(100% - 1em)}.dark #ModelSummaryPage_modelSummaryPage__w9xEy{background:#111}